# -*- coding: utf-8 -*-
# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
# 
# This module is under the UIUC open-source license. See 
# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
# for license details.
"""
"""
from biosteam.process_tools import UnitGroup
from biorefineries import corn as cn
from biorefineries.sugarcane import ConventionalEthanolTEA

__all__ = ('create_tea', 'tea_summary')

def create_tea(system, cls=ConventionalEthanolTEA):
    return cls(system, IRR=0.15,
               duration=(2018, 2038),
               depreciation='MACRS7', income_tax=0.35,
               operating_days=330, lang_factor=4,
               construction_schedule=(0.4, 0.6), WC_over_FCI=0.05,
               labor_cost=2.3e6, fringe_benefits=0.4,
               property_tax=0.001, property_insurance=0.005,
               supplies=0.20, maintenance=0.01, administration=0.005)

def tea_summary():
    operating_days = cn.corn_tea.operating_days
    f = operating_days * 24
    ug = UnitGroup('biorefinery', cn.corn_tea.units)
    power_utilities = ug.power_utilities
    heat_utilities = ug.heat_utilities
    heating_utilities = [i for i in heat_utilities if i.duty > 0]
    cooling_utilities = [i for i in heat_utilities if i.duty < 0]
    FCI = cn.corn_tea.FCI
    FOC = cn.corn_tea.FOC
    maintenance = cn.corn_tea.maintenance * FCI
    
    dct = {
        'Material costs':
            {'Corn': f * cn.corn.cost ,
             'Denaturant': f * cn.denaturant.cost,
             'Enzymes': f * (cn.alpha_amylase.cost + cn.gluco_amylase.cost),
             'Yeast': f * cn.yeast.cost,
             'Other': f * sum([i.cost for i in (cn.ammonia, cn.sulfuric_acid, cn.lime)])},
         'Production':
             {'Ethanol [MMgal / yr]': f * cn.ethanol.F_mass / 2.987 / 1e6,
              'DDGS [MT / yr]': f * cn.DDGS.F_mass / 1e3,
              'Crude oil [MT / yr]': f * cn.crude_oil.F_mass / 1e3},
         'Sales':
             {'Ethanol': f * cn.ethanol.cost,
              'DDGS': f * cn.DDGS.cost,
              'Crude oil': f * cn.crude_oil.cost},
         'Utilities':
             {'Electricity': f * sum([i.cost for i in power_utilities]),
              'Steam': f * (sum([i.cost for i in heating_utilities]) + cn.steam.cost),
              'Natural gas': f * cn.D610.natural_gas_cost,
              'Cooling water': f * sum([i.cost for i in cooling_utilities])},
         'Labor and supplies': 
             {'Plant operations': cn.corn_tea.labor_cost * (1. + cn.corn_tea.fringe_benefits + cn.corn_tea.supplies),
              'Maintenance': maintenance},
         'Insurance, property tax, and administration':
             FCI*(cn.corn_tea.property_tax + cn.corn_tea.property_insurance + cn.corn_tea.administration),
         
         'Depreciation': cn.corn_tea.TDC / 10.,
         'Fixed operating cost': FOC,
         'Variable operating cost': cn.corn_tea.VOC,
         'Fixed capital cost': FCI,
         
    }
    sales = dct['Sales']
    dct['Co-product credit'] = credit = - sales['DDGS'] - sales['Crude oil']
    dct['Total production cost'] = x = sum([*dct['Material costs'].values(),
                                        *dct['Utilities'].values(),
                                        dct['Depreciation'],
                                        dct['Co-product credit'],
                                        dct['Fixed operating cost']]) 
    dct['Production cost [USD/gal]'] = x / dct['Production']['Ethanol [MMgal / yr]'] / 1e6
    return dct


# -*- coding: utf-8 -*-
"""
This module exposes the `ABM_TEA_model`, a biosteam.Model object
for evaluating multiple scenarios for the agent based model project.

Examples
--------
>>> import numpy as np
>>> from biorefineries.corn.abm import ABM_TEA_model
>>> corn_price = [0.0847, 0.09] # Corn price [USD/kg]
>>> DDGS_price = [0.05159, 0.0968] # DDGS price [USD/kg]
>>> corn_oil_price = [0.5, 0.6] # Corn oil price [USD/kg]
>>> ethanol_price = [0.485, 0.5] # Ethanol price [USD/kg]
>>> operating_days = [330., 330.] # Operating days [day/yr]
>>> IRR = [0.1, 0.15] # Internal rate of return
>>> start_year = [2020, 2020]
>>> end_year = [2040, 2040]
>>> plant_capacity = [876072883, 876072883] # Plant capacity [kg/yr]
>>> samples = np.array([
...     corn_price, 
...     DDGS_price,
...     corn_oil_price,
...     ethanol_price,
...     operating_days,
...     IRR,
...     start_year,
...     end_year,
...     plant_capacity,
... ]).T
>>> ABM_TEA_model.load_samples(samples)
>>> ABM_TEA_model.evaluate()
>>> table = ABM_TEA_model.table # A pandas data frame
>>> MESP_index = ABM_TEA_model.metrics[0].index
>>> ABM_TEA_model.table[MESP_index] 
0   0.385
1   0.386
Name: (Biorefinery, MESP [USD/kr]), dtype: float64
>>> parameters = ABM_TEA_model.get_parameters()
>>> cornstover_price_index = parameters[0].index
>>> ABM_TEA_model.table[cornstover_price_index] 
0   0.0847
1     0.09
Name: (Corn, Price [USD/kg]), dtype: float64

"""
from biorefineries import corn as cn
import biosteam as bst

__all__ = ('ABM_TEA_model',)

# %% Functional ABM TEA model for legacy purposes

def ABM_TEA_function(
        operating_days=330,
        plant_capacity=876072883.4242561, 
        price_corn=0.13227735731092652, 
        price_DDGS=0.12026, 
        price_corn_oil=0.56,
        price_ethanol=0.48547915353569393,
        IRR=0.15,
        duration=(2007, 2027),
    ):
    """
    Return a dictionary of biorefinery metrics for the production of cellulosic
    ethanol from mixed feedstocks.

    Parameters
    ----------
    operating_days : float
        Number of operating days per year.
    plant_capacity : float
        Plat capacity in kg/yr of feedstock.
    price_corn : float
        Price of corn in USD/kg.
    price_DDGS : float
        Price of DDGS in USD/kg.
    price_ethanol : float
        Price of ethanol in USD/kg.
    IRR : float
        Internal rate of return as a fraction (not percent!).
    duration : tuple(int, int)
        Years of operation.

    Returns
    -------
    metrics: dict
        Includes MESP [USD/kg], MFPP [USD/kg], IRR [-], NPV [USD], 
        TCI [USD], FOC [USD/yr], VOC [USD/yr], Electricity consumption [MWhr/yr], 
        Electricity production [MWhr/yr], and Production [kg/yr].
    
    """
    cn.corn.price = price_corn
    cn.DDGS.price = price_DDGS
    cn.crude_oil.price = price_corn_oil
    cn.ethanol.price = price_ethanol
    hours = operating_days * 24 
    cn.corn.F_mass = plant_capacity / hours
    cn.corn_tea.operating_days = operating_days
    cn.corn_tea.duration = duration
    cn.corn_sys.simulate()
    cn.corn_tea.IRR = IRR
    unit_group = cn.all_areas
    return {
        'MESP': cn.corn_tea.solve_price(cn.ethanol),
        'MFPP': cn.corn_tea.solve_price(cn.corn),
        'IRR': cn.corn_tea.solve_IRR(),
        'NPV': cn.corn_tea.NPV,
        'TCI': cn.corn_tea.TCI,
        'VOC': cn.corn_tea.VOC,
        'FOC': cn.corn_tea.FOC,
        'Electricity consumption [MWhr/yr]': hours * unit_group.get_electricity_consumption(), 
        'Electricity production [MWhr/yr]': hours * unit_group.get_electricity_production(),
        'Production': hours * cn.ethanol.F_mass,
    }

# %% ABM Model object

metrics = [
    bst.Metric('MESP', lambda:cn.corn_tea.solve_price(cn.ethanol), 'USD/kr'),    
    bst.Metric('MFPP', lambda:cn.corn_tea.solve_price(cn.corn), 'USD/kr'),
    bst.Metric('IRR', cn.corn_tea.solve_IRR),
    bst.Metric('NPV', lambda:cn.corn_tea.NPV, 'USD'),
    bst.Metric('TCI', lambda:cn.corn_tea.TCI, 'USD'),
    bst.Metric('VOC', lambda:cn.corn_tea.VOC, 'USD/yr'),
    bst.Metric('FOC', lambda:cn.corn_tea.FOC, 'USD/yr'),
    bst.Metric('Electricity consumption', lambda:operating_hours * cn.all_areas.get_electricity_consumption(), 'MWhr/yr'),
    bst.Metric('Electricity production', lambda:operating_hours * cn.all_areas.get_electricity_production(), 'MWhr/yr'),
    bst.Metric('Production', lambda:operating_hours * cn.ethanol.F_mass, 'kg/yr')
]

ABM_TEA_model = bst.Model(cn.corn_sys, metrics)

@ABM_TEA_model.parameter(element='Corn', units='USD/kg')
def set_corn_price(price):
    cn.corn.price = price

@ABM_TEA_model.parameter(element='DDGS', units='USD/kg')
def set_DDGS_price(price):
    cn.DDGS.price = price
    
@ABM_TEA_model.parameter(element='Corn oil', units='USD/kg')
def set_corn_oil_price(price):
    cn.crude_oil.price = price

@ABM_TEA_model.parameter(element='Ethanol', units='USD/kg')
def set_ethanol_price(price):
    cn.ethanol.price = price

operating_hours = cn.corn_tea.operating_days * 24
@ABM_TEA_model.parameter(units='day/yr')
def set_operating_days(operating_days):
    global operating_hours
    cn.corn_tea.operating_days = operating_days
    operating_hours = operating_days * 24

@ABM_TEA_model.parameter
def set_IRR(IRR):
    cn.corn_tea.IRR = IRR

@ABM_TEA_model.parameter
def set_start_year(start_year):
    cn.corn_tea.duration = (start_year, cn.corn_tea.duration[1])
    
@ABM_TEA_model.parameter
def set_end_year(end_year):
    cn.corn_tea.duration = (cn.corn_tea.duration[0], end_year)

@ABM_TEA_model.parameter(units='kg/yr')
def set_plant_capacity(plant_capacity):
    cn.corn.F_mass = plant_capacity / operating_hours



# -*- coding: utf-8 -*-
# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
# 
# This module is under the UIUC open-source license. See 
# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
# for license details.
"""
"""

__all__ = ('dextrose_equivalent',)

def dextrose_equivalent(n):
    """
    Return the dextrose equivalent of starch given the degree of polymerization.

    Parameters
    ----------
    n : int
        Degree of polymerization.

    Returns
    -------
    DE : float
        Dextrose equivalent.

    Notes
    -----
    The dextrose equivalent (DE) is a measure of the amount of reducing sugars 
    present in a sugar product, expressed as a percentage on a dry basis 
    relative to dextrose. For polymerized glucose (i.e. dissacharides, 
    oligosaccharides, and polysaccharides), the dextrose equivalent is given 
    by the following formula:
        
    .. math::
        DE = 100% \frac{180}{180n - 18(n - 1)}
        
    This formula (and this function) is not valid for sugars with linkages other 
    than 1-4 and 1-6 glycosidic linkages.
    
    """
    if n < 1: 
        raise ValueError('degree of polymerization, n, must be greater or equal to 1')
    n = float(n)
    MW_glucose = 180.
    MW_water = 18.
    return 100. * MW_glucose / (MW_glucose * n - MW_water * (n - 1))


# -*- coding: utf-8 -*-
# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
# 
# This module is under the UIUC open-source license. See 
# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
# for license details.
"""
"""
import os
import biosteam as bst
import thermosteam as tmo
from biosteam import units
from thermosteam.reaction import Reaction as Rxn
from biorefineries.fattyalcohols import units as fa_units

__all__ = ('create_fattyalcohol_production_sys',)


# TODO: 
# - Add CSL and DAP as nutrients (remove tryptone)
# - Include stoichiometry similar to Humbird et. al.
# - Include salt price.

# Nitrogen requirement:
# - 0.0444 g N / g CSL (0.0725 mol N / mol CSL)
# - 0.212 g N / g DAP (2 mol N / mol DAP)
# - 0.0902 g N / g Cells (0.16 mol N / mol Cell)
# - 0.7362 g CSL / g DAP (2.611 mol CSL / mol DAP)

# Growth equation: 
# Let x be # mol DAP and y be # mol Cells
# - Mass balance:
#   MW_glucose + x * (2.611 * MW_csl + MW_dap) = y * MW_cells + 2 * MW_co2 + x * 1.3055 * MW_h2o + x * MW_h3po4
#   y = 4.3398 * x  + 3.71
# - Nitrogen balance:
#   x * (2.611 * 0.0725 + 2) = y * 0.16
#   y = 13.68 * x
# - Solve:
#   4.3398 * x  + 3.71 = 13.68 * x
#   x = 0.3972
#   y = 5.4338
# - Equation:
#   Glucose + 1.037 CSL + 0.3972 DAP ->  5.4338 Cells + 2 CO2 + 0.51858 H2O + 0.3972 H3PO4

# Bench scale stoichiometry: 
# Glucose -> 1.5 Hexanol + 1.5 Octanol + 4.5 Decanol + 
#            6 Dodecanol + 4 Tetradecanol + H2O + CO2
# Remove heavier alcohol products to ease separations
def create_fattyalcohol_production_sys(ID='fattyalcohol_production_sys',
    fa_stoichiometry='Glucose -> 1.5 Hexanol + 1.5 Octanol + 4.5 Decanol + H2O + CO2',
    yg_stoichiometry='Glucose ->  3.72 Cells + 2 CO2'):
    
    chemicals = tmo.settings.get_chemicals()
    
    ### Reactions ###

    fatty_alcohol_production = Rxn(fa_stoichiometry, 'Glucose', 0.5, basis='wt')
    fatty_alcohol_production.basis = 'mol'
    fatty_alcohol_production.correct_atomic_balance(
        constants = ['Hexanol', 'Octanol', 'Decanol', 'Dodecanol', 'Tetradecanol'])
    yeast_growth = Rxn(yg_stoichiometry, 'Glucose', .999, correct_mass_balance=True)
    
    ### Streams ###
    filepath = os.path.dirname(__file__)
    stream_data_path = os.path.join(filepath, 'streams.yaml')   
    stream_data = tmo.ThermoData.from_yaml(stream_data_path)
    glucose, process_water, tridecane, CSL, DAP, salt, \
    cell_recycle, mixed_bioreactor_feed = stream_data.create_streams(
        ['glucose', 'process_water', 'tridecane', 'CSL', 'DAP',
         'salt', 'cell_recycle', 'mixed_bioreactor_feed']
    )
    process_water.T = 310.15 # Assume heat integration so that this stream comes in heated

    ### Unit operations ###
    
    M101 = units.Mixer('M101', ins=(tridecane, 'solvent_recycle'))
    M101.ins[1].T = 310.15 # Assume heat integration so that this stream comes in heated
    
    one_week = 7*24
    T101 = units.StorageTank('T101', glucose, tau=one_week)
    T102 = units.StorageTank('T102', M101-0, tau=0.25)
    P102 = units.Pump('P102', T102-0)
    T103 = fa_units.CSLStorageTank('T103', CSL)
    T104 = units.StorageTank('T104', salt, tau=one_week)
    T106 = fa_units.DAPStorageTank('T106', DAP)
        
    def balance_feeds_to_bioreactor():
        feed_imol = mixed_bioreactor_feed.imol
        glucose.imol['Glucose'] = feed_imol['Glucose']
        process_water.imol['Water'] = feed_imol['Water'] - M101.ins[1].imol['Water']
        tridecane.imol['Tridecane'] = feed_imol['Tridecane'] - M101.ins[1].imol['Tridecane']
        salt.imol['NaCl'] = feed_imol['NaCl']
        cell_recycle.imol['Cells'] = feed_imol['Cells']
        CSL.imol['CSL'] = feed_imol['CSL']
        DAP.imol['DAP'] = feed_imol['DAP']
        M101._run()
    
    M101.specification = balance_feeds_to_bioreactor
    M102 = units.Mixer('M102',
                       ins=(T101-0, P102-0, T103-0, T104-0, T106-0,
                            process_water, cell_recycle))
    H101 = units.HXutility('H101',
                            ins=M102-0,
                            outs=mixed_bioreactor_feed,
                            material='Stainless steel/stainless steel',
                            T=310.15, V=0)
    def make_sure_enough_CSL_and_DAP_are_fed():
        # tmo.reaction.CHECK_FEASIBILITY = False
        feed = R101.ins[0]
        CSL.imass['CSL'] = feed.imass['CSL'] = 0.0025 * feed.F_mass
        DAP.imass['DAP'] = feed.imass['DAP'] = 0.33 * feed.F_vol
        R101._run()
        # effluent = R101.outs[1]
        # if effluent.imol['CSL'] < 0. or effluent.imol['DAP'] < 0.:
        #     CSL.imol['CSL'] = feed.imol['CSL'] = CSL_flow - effluent.imol['CSL']
        #     DAP.imol['DAP'] = feed.imol['DAP'] = DAP_flow - effluent.imol['DAP']
        #     effluent.imol['CSL'] = effluent.imol['DAP'] = 0.
        # tmo.reaction.CHECK_FEASIBILITY = True    
    R101 = fa_units.FattyAlcoholBioreactor('R101', outs=('CO2', ''), ins=H101-0,
                                     fermentation_reaction=fatty_alcohol_production,
                                     postfermentation_reaction=yeast_growth, T=310.15, tau=32, 
                                     V=3785, Nmin=2, Nmax=32)
    R101.specification = make_sure_enough_CSL_and_DAP_are_fed
    T105 = units.StorageTank('T105', ins=R101-1, tau=2)
    solids_split = chemicals.kwarray({'Cells': 0.999})
    solids_split[solids_split == 0] = 0.001
    solids_split[chemicals.index('Water')] = 0
    P104 = units.Pump('P104', ins=T105-0)
    C101 = fa_units.SolidLiquidsSplitCentrifuge('C101',
            ins=P104-0, outs=('', 'aqueous_fraction', 'cell_mass'),
            solids_split=solids_split,
            liquids_split=dict(
                Water=0.0004,
                Tridecane=0.9966,
                Hexanol=0.6781,
                Octanol=0.9608,
                Decanol=0.9966,
                Dodecanol=0.9999,
                Tetradecanol=1.0),
            moisture_content=0.45)
    P107 = units.Pump('P107', C101-0, 'oil_fraction', P=5*101325)
    
    ### System ###
    
    return bst.System(ID, (M101, T102, P102, T101, T104, T103, T106, M102, H101,
                           R101, T105, P104, C101, P107))




# -*- coding: utf-8 -*-
"""
This module contains legacy code for the cellulosic ethanol biorefinery
based on the 2011 NREL report [1]_.

References
----------
.. [1] Humbird, D., Davis, R., Tao, L., Kinchin, C., Hsu, D., Aden, A.,
    Dudgeon, D. (2011). Process Design and Economics for Biochemical 
    Conversion of Lignocellulosic Biomass to Ethanol: Dilute-Acid 
    Pretreatment and Enzymatic Hydrolysis of Corn Stover
    (No. NREL/TP-5100-47764, 1013269). https://doi.org/10.2172/1013269

"""

# %% Legacy code

### DO NOT DELELTE:
### This code is used to generate splits for unit operations in wastewater 
### treatment of a cellulosic ethanol biorefinery.
import thermosteam as tmo
import numpy as np
from biorefineries.cornstover import chemicals
tmo.settings.set_thermo(chemicals)
chemical_groups = dict(
        OtherSugars = ('Arabinose',
                        'Mannose',
                        'Galactose',
                        'Cellobiose',
                        'Sucrose'),
        SugarOligomers = ('GlucoseOligomer',
                          'XyloseOligomer',
                          'GalactoseOligomer',
                          'ArabinoseOligomer',
                          'MannoseOligomer'),
        OrganicSolubleSolids = ('AmmoniumAcetate',
                                'SolubleLignin',
                                'Extract', 
                                'LacticAcid', 
                                'Cellulase'),
        InorganicSolubleSolids = ('AmmoniumSulfate',
                                  'DAP',
                                  'NaOH',
                                  'HNO3',
                                  'NaNO3'),
        Furfurals = ('Furfural',
                      'HMF'),
        OtherOrganics = ('Glycerol',
                          'Denaturant',
                          'Oil',
                          'SuccinicAcid',
                          'Xylitol'),
        COxSOxNOxH2S = ('NO',
                        'NO2',
                        'SO2',
                        'CO',
                        'H2S'),
        Protein = ('Protein',
                    'Enzyme',
                    'DenaturedEnzyme'),
        CellMass = ('WWTsludge',
                    'Z_mobilis',
                    'T_reesei'),
        OtherInsolubleSolids = ('Tar',
                                'Ash',
                                'Lime'),
        OtherStructuralCarbohydrates = ('Arabinan', 
                                        'Mannan', 
                                        'Galactan')
)

def find_split(IDs, flow0, flow1):
    flow0 = np.asarray(flow0)
    splits = flow0/(flow0 + np.asarray(flow1))
    chemicals = tmo.settings.get_chemicals()
    array = np.zeros(chemicals.size)
    for ID, split in zip(IDs, splits):
        if ID in chemical_groups:
            array[chemicals.get_index(chemical_groups[ID])] = split
        else:
            array[chemicals.index(ID)] = split
    return array

splits = [
    ('Ethanol', 1, 15),
    ('Water', 27158, 356069),
    ('Glucose', 3, 42),
    ('Xylose', 7, 85),
    ('OtherSugars', 13, 175),
    ('SugarOligomers', 10, 130),
    ('OrganicSolubleSolids', 182, 2387),
    ('InorganicSolubleSolids', 8, 110),
    ('Ammonia', 48, 633),
    ('AceticAcid', 0, 5),
    ('Furfurals', 5, 70),
    ('OtherOrganics', 9, 113),
    ('Cellulose', 19, 6),
    ('Xylan', 6, 2),
    ('OtherStructuralCarbohydrates', 1, 0),
    ('Lignin', 186, 64),
    ('Protein', 51, 18),
    ('CellMass', 813, 280),
    ('OtherInsolubleSolids', 68, 23)
]

anaerobic_bioreactor_sludge_splits = find_split(*zip(*splits))

splits = [
    ('Glucose', 19, 502),
    ('Xylose', 40, 1022),
    ('OtherSugars', 81, 2175),
    ('SugarOligomers', 60, 1552),
    ('OrganicSolubleSolids', 612, 15808),
    ('InorganicSolubleSolids', 97, 2513),
    ('Furfurals', 19, 513),
    ('OtherOrganics', 52, 1348),
    ('Glucan', 1230, 25),
    ('Xylan', 415, 8),
    ('OtherStructuralCarbohydrates', 94, 2),
    ('Lignin', 12226, 250),
    ('Protein', 3376, 69),
    ('CellMass', 925, 19),
    ('OtherInsolubleSolids', 4489, 92)
]

pressure_filter_splits = find_split(*zip(*splits))

splits = [
    ('Ethanol', 0, 1),
    ('Water', 381300, 2241169),
    ('Glucose', 0, 2),
    ('Xylose', 1, 3),
    ('OtherSugars', 1, 7),
    ('SugarOligomers', 1, 6),
    ('OrganicSolubleSolids', 79, 466),
    ('InorganicSolubleSolids', 4828, 28378),
    ('Ammonia', 3, 16),
    ('Furfurals', 0, 3),
    ('OtherOrganics', 1, 7),
    ('CarbonDioxide', 6, 38),
    ('O2', 3, 17),
    ('N2', 5, 32),
    ('Cellulose', 0, 194),
    ('Xylan', 0, 65),
    ('OtherStructuralCarbohydrates', 0, 15),
    ('Lignin', 0, 1925),
    ('Protein', 0, 90),
    ('CellMass', 0, 19778),
    ('OtherInsolubleSolids', 0, 707)
]

membrane_bioreactor_splits = find_split(*zip(*splits))

centrifuge_species = ('Water', 'Glucose', 'Xylose', 'OtherSugars',
                      'SugarOligomers', 'OrganicSolubleSolids',
                      'InorganicSolubleSolids', 'Ammonia', 'Furfurals', 
                      'OtherOrganics', 'CO2', 'COxSOxNOxH2S', 'Cellulose',
                      'Xylan', 'OtherStructuralCarbohydrates', 'Lignin',
                      'Protein', 'CellMass', 'OtherInsolubleSolids')
S623_flow = np.array([7708, 0, 0, 1, 1, 13, 75, 3, 0, 1, 1, 2, 25, 8, 2, 250, 52, 1523, 92])
S616_flow = np.array([109098, 3, 6, 13, 9, 187, 1068, 46, 5, 8, 14, 31, 1, 0, 0, 13, 3, 80, 5])

sludge_centrifuge_splits = find_split(centrifuge_species, S616_flow, S623_flow)